from .reqrep import RequestHandler
from .util import format_url

__all__ = ['Forecast', 'ForecastDataset']


class Forecast:

    def __init__(self, api_key, *, timeout=None, connector=None, loop=None):
        """
        Parameters:
            api_key (str): your Dark Sky API key
        """
        self.api_key = api_key
        self.req_handler = RequestHandler(timeout, connector, loop=loop)

    async def call(self, latitude, longitude, *, time=None, **kwargs):
        """

        Parameters:
            latitude (float): geographic coordinate of location
                              in decimal degrees
            longitude (float): geographic coordinate of location
                               in decimal degrees
            time (object): should either be a UNIX time or a string formatted
                           as follows: [YYYY]-[MM]-[DD]T[HH]:[MM]:[SS]
                           (with an optional time zone formatted as Z for GMT
                           time or {+,-}[HH][MM] for an offset in hours or
                           minutes).
                           For the latter format, if no timezone is present,
                           local time (at the provided latitude and longitude)
                           is assumed. (This string format is a subset of ISO
                           8601 time. An as example, 2013-05-06T12:00:00-0400.)
            **kwargs: Their are plenty of options, like units, exclude, etc.
                      You should refer to the official documentation for a
                      full description of them.
        """
        url = format_url(self.api_key, latitude, longitude, time)
        params = {k: format_param(v) for k, v in kwargs.items()}
        response = await self.req_handler.get(url, params=params)
        return ForecastDataset(response.data, response.headers)

    def close(self):
        self.req_handler.close()

    def __del__(self):
        self.close()


def format_param(data):
    if isinstance(data, (list, set, tuple)):
        return ','.join(str(d) for d in data)
    return str(data)


class ForecastDataset:

    def __init__(self, data, headers):
        self._data = data
        self._headers = headers

    def __getattr__(self, name):
        try:
            return self._data[name]
        except KeyError as error:
            raise AttributeError('%s is not defined' % name) from error
